==> makatto-0.0.1.tgz: package/lib/server.js <==
var http  = require("http");
var fs    = require("fs");
var wsAPI = require("websocket.io");
var path  = require("path");
var url   = require("url");

var Server = function(port, httpRoot, debug) {
	var self      = this;
	
	this.contentTypes = {
		".js"		: "text/javascript",
		".css"		: "text/css",
		".png"		: "image/png",
		".gif"		: "image/gif",
		".jpg"		: "image/jpeg",
		"default"	: "text/html"
	};
	
	this.customRequests = {
		"/makatto-reload.js" : this._customRequestReload
	};
	
	this.port     = port;
	this.httpRoot = httpRoot;
	this.debug    = !!debug;
	
	this.server   = this._createServer();
	this.ws       = this._createWS();
	this.socket   = null;
};

Server.prototype = {
	
	start : function(callback) {
		var self = this;
		
		this.server.listen(this.port, function() {
			callback && callback();
		});
	},
	
	stop : function(callback) {
		this.socket && this.socket.close();
		this.server.close(callback);
	},
	
	_createServer : function() {
		var self = this;
		return http.createServer(function(request, response) {
			self._onRequest.apply(self, arguments);
		});
	},
	
	_createWS : function() {
		var self = this;
		
		var ws = wsAPI.attach(this.server);
		ws.on("connection", function(_socket) {
			self._onWsConnect.apply(self, arguments);
		});
	},
	
	_onRequest : function(request, response) {
		var self = this;
		var filePath;
		var urlParts = url.parse(request.url, true);
		
		if (urlParts.pathname === "/favicon.ico") {
			return;
		}
		
		if (urlParts.pathname === "/") {
			response.writeHead(301, {
				"Location" : "/index.html"
			});
			response.end();
			console.log("301 - / -> /index.html");
			return;
		}
		else {
			if (self.customRequests[urlParts.pathname]) {
				self.customRequests[urlParts.pathname].call(self, urlParts, request, response);
				return;
			}
			else {
				filePath = self.httpRoot + "/" + urlParts.pathname;
			}
		}
		
		fs.exists(filePath, function(exists) {
			var ext = path.extname(filePath);
			
			if (!exists) {
				response.writeHead(404);
				response.end("404", "utf-8");
				console.log("404 - " + request.url);
				return;
			}
			fs.readFile(filePath, function(error, content) {
				if (error) {
					response.writeHead(500);
					response.end("500", "utf-8");
					console.log("500 - " + request.url);
					return;
				}
				
				if (ext === ".html" && self.debug === true) {
					content = content.toString().replace("<\/head>", "\t<script src=\"/makatto-reload.js\"><\/script>\n<\/head>");
				}
				
				response.writeHead(200, {
					"Content-Type" : self.contentTypes[ext] || self.contentTypes["default"] + ";charset=UTF-8",
					"Cache-Control" : "no-cache",
					"Expires" : "-1"
				});
				response.end(content, "utf-8");
				console.log("200 - " + request.url);
			});
		});
	},
	
	_customRequestReload : function(urlParts, request, response) {
		var script = fs.readFileSync(__dirname + "/reload.js", "utf-8");
		script = script.replace("%%PORT%%", this.port);
		script = script.replace("%%HOST%%", "localhost");
		
		response.writeHead(200, {
			"Content-Type" : "text/javascript;charset=UTF-8",
			"Cache-Control" : "no-cache",
			"Expires" : "-1"
		});
		response.end(script, "utf-8");
		console.log("200 - " + request.url);
	},
	
	_onWsConnect : function(_socket) {
		var self = this;
		
		this.socket = _socket;
		this.socket.on("message", function(msg) {
			self._onWsMessage.apply(self, arguments);
		});
		this.socket.on("close", function() {
			self.socket = null;
		});
	},
	
	_onWsMessage : function(msg) {
		console.log("WS< " + msg);
	}
	
};

exports.Server = Server;

==> makatto-1.1.1.tgz: package/lib/server.js <==
var http  = require("http");
var fs    = require("fs");
var wsAPI = require("websocket.io");
var path  = require("path");
var url   = require("url");

var Server = function(port, httpRoot, livereload) {
	var self      = this;
	
	this.contentTypes = {
		".js"		: "text/javascript",
		".css"		: "text/css",
		".png"		: "image/png",
		".gif"		: "image/gif",
		".jpg"		: "image/jpeg",
		"default"	: "text/html"
	};
	
	this.customRequests = {
		"/makatto-reload.js" : this._customRequestReload
	};
	
	this.port     = port;
	this.httpRoot = httpRoot;
	this.livereload    = !!livereload;
	
	this.server   = this._createServer();
	this.ws       = this._createWS();
	this.socket   = null;
};

Server.prototype = {
	
	start : function(callback) {
		var self = this;
		
		this.server.listen(this.port, function() {
			callback && callback();
		});
	},
	
	stop : function(callback) {
		this.socket && this.socket.close();
		this.server.close(callback);
	},
	
	_createServer : function() {
		var self = this;
		return http.createServer(function(request, response) {
			self._onRequest.apply(self, arguments);
		});
	},
	
	_createWS : function() {
		var self = this;
		
		var ws = wsAPI.attach(this.server);
		ws.on("connection", function(_socket) {
			self._onWsConnect.apply(self, arguments);
		});
	},
	
	_onRequest : function(request, response) {
		var self = this;
		var filePath;
		var urlParts = url.parse(request.url, true);
		
		if (urlParts.pathname === "/favicon.ico") {
			return;
		}
		
		if (urlParts.pathname === "/") {
			response.writeHead(301, {
				"Location" : "/index.html"
			});
			response.end();
			console.log("301 - / -> /index.html");
			return;
		}
		else {
			if (self.customRequests[urlParts.pathname]) {
				self.customRequests[urlParts.pathname].call(self, urlParts, request, response);
				return;
			}
			else {
				filePath = self.httpRoot + "/" + urlParts.pathname;
			}
		}
		
		fs.exists(filePath, function(exists) {
			var ext = path.extname(filePath);
			
			if (!exists) {
				response.writeHead(404);
				response.end("404", "utf-8");
				console.log("404 - " + request.url);
				return;
			}
			fs.readFile(filePath, function(error, content) {
				if (error) {
					response.writeHead(500);
					response.end("500", "utf-8");
					console.log("500 - " + request.url);
					return;
				}
				
				if (ext === ".html" && self.livereload === true) {
					content = content.toString().replace("<\/head>", "\t<script src=\"/makatto-reload.js\"><\/script>\n<\/head>");
				}
				
				response.writeHead(200, {
					"Content-Type" : self.contentTypes[ext] || self.contentTypes["default"] + ";charset=UTF-8",
					"Cache-Control" : "no-cache",
					"Expires" : "-1"
				});
				response.end(content, "utf-8");
				console.log("200 - " + request.url);
			});
		});
	},
	
	_customRequestReload : function(urlParts, request, response) {
		var script = fs.readFileSync(__dirname + "/reload.js", "utf-8");
		script = script.replace("%%PORT%%", this.port);
		script = script.replace("%%HOST%%", "localhost");
		
		response.writeHead(200, {
			"Content-Type" : "text/javascript;charset=UTF-8",
			"Cache-Control" : "no-cache",
			"Expires" : "-1"
		});
		response.end(script, "utf-8");
		console.log("200 - " + request.url);
	},
	
	_onWsConnect : function(_socket) {
		var self = this;
		
		this.socket = _socket;
		this.socket.on("message", function(msg) {
			self._onWsMessage.apply(self, arguments);
		});
		this.socket.on("close", function() {
			self.socket = null;
		});
	},
	
	_onWsMessage : function(msg) {
		console.log("WS< " + msg);
	}
	
};

exports.Server = Server;
